@@ -20,7 +20,7 @@
 \********************************************************************/
 
 /** @file const.h
-    @brief xfrp constant parameter define
+    @brief xfrpc constant parameter define
     @author Copyright (C) 2016 Dengfeng Liu <liu_df@qq.com>
 */
 
@@ -34,15 +34,5 @@ enum server_status {
 	Closed
 };
 
-// msg type
-// enum msg_type {
-// 	NewCtlConn = 0,
-// 	NewWorkConn,
-// 	NoticeUserConn,
-// 	NewCtlConnRes,
-// 	HeartbeatReq,
-// 	HeartbeatRes,
-// 	NewWorkConnUdp
-// };
 
 #endif
