@@ -34,14 +34,7 @@
 #include <netinet/in.h>
 #include <json-c/json.h>
 #include <syslog.h>
-
-#include <event2/bufferevent.h>
-#include <event2/buffer.h>
-#include <event2/listener.h>
-#include <event2/util.h>
-#include <event2/event.h>
-#include <event2/dns.h>
-#include <event2/event_struct.h>
+#include <unistd.h>
 
 #include "debug.h"
 #include "client.h"
@@ -53,16 +46,17 @@
 #include "uthash.h"
 #include "crypto.h"
 #include "utils.h"
-#include "session.h"
 #include "common.h"
 #include "login.h"
+#include "tcpmux.h"
 
 static struct control *main_ctl;
 static int clients_conn_signel = 0;
 static int is_login = 0;
 
-static void sync_new_work_connection(struct bufferevent *bev);
+static void sync_new_work_connection(struct bufferevent *bev, uint32_t sid);
 static void recv_cb(struct bufferevent *bev, void *ctx);
+static void clear_main_control();
 
 static int 
 is_client_connected()
@@ -108,11 +102,11 @@ client_start_event_cb(struct bufferevent *bev, short what, void *ctx)
 		}
 		debug(LOG_ERR, "Proxy connect server [%s:%d] error: %s", c_conf->server_addr, c_conf->server_port, strerror(errno));
 		bufferevent_free(bev);
-		free_proxy_client(client);
+		del_proxy_client(client);
 	} else if (what & BEV_EVENT_CONNECTED) {
 		bufferevent_setcb(bev, recv_cb, NULL, client_start_event_cb, client);
 		bufferevent_enable(bev, EV_READ|EV_WRITE);
-		sync_new_work_connection(bev);
+		sync_new_work_connection(bev, 0);
 		client_connected(1);
 		debug(LOG_INFO, "proxy service start");
 	}
@@ -125,6 +119,14 @@ new_client_connect()
 	struct common_conf *c_conf = get_common_config();
 	assert(c_conf);
 	client->base = main_ctl->connect_base;
+	
+	if (c_conf->tcp_mux) {
+		debug(LOG_DEBUG, "new client through tcp mux: %d", client->stream_id);
+		client->ctl_bev 	= main_ctl->connect_bev;
+		sync_new_work_connection(client->ctl_bev, client->stream_id);
+		return;
+	}
+
 	struct bufferevent *bev = connect_server(client->base, c_conf->server_addr, c_conf->server_port);
 	if (!bev) {
 		debug(LOG_DEBUG, "Connect server [%s:%d] failed", c_conf->server_addr, c_conf->server_port);
@@ -142,10 +144,13 @@ static void
 start_proxy_services()
 {
 	struct proxy_service *all_ps = get_all_proxy_services();
-	assert(all_ps);
-
 	struct proxy_service *ps = NULL, *tmp = NULL;
 	
+	if (!all_ps) {
+		debug(LOG_INFO, "no proxy service configure by user");
+		return;
+	}
+
 	debug(LOG_INFO, "Start xfrp proxy services ...");
 	
 	HASH_ITER(hh, all_ps, ps, tmp) {
@@ -158,27 +163,21 @@ start_proxy_services()
 }
 
 static void 
-ping(struct bufferevent *bev)
+ping()
 {
-	struct bufferevent *bout = NULL;
-	if (bev) {
-		bout = bev;
-	} else {
-		bout = main_ctl->connect_bev;
-	}
+	struct bufferevent *bout = main_ctl->connect_bev;
 
 	if ( ! bout) {
 		debug(LOG_ERR, "bufferevent is not legal!");
 		return;
 	}
 	
-	uint32_t sid = get_main_control()->session_id;
 	char *ping_msg = "{}";
-	send_enc_msg_frp_server(bev, TypePing, ping_msg, strlen(ping_msg), sid);
+	send_enc_msg_frp_server(bout, TypePing, ping_msg, strlen(ping_msg), main_ctl->stream_id);
 }
 
 static void 
-sync_new_work_connection(struct bufferevent *bev)
+sync_new_work_connection(struct bufferevent *bev, uint32_t sid)
 {
 	struct bufferevent *bout = bev;
 	assert(bout);
@@ -197,8 +196,10 @@ sync_new_work_connection(struct bufferevent *bev)
 		debug(LOG_ERR, "new work connection request run_id marshal failed!");
 		return;
 	}
+	
+	tcp_mux_send_win_update_syn(bev, sid);
 
-	send_msg_frp_server(bev, TypeNewWorkConn, new_work_conn_request_message, nret, 0);
+	send_msg_frp_server(bev, TypeNewWorkConn, new_work_conn_request_message, nret, sid);
 
 	SAFE_FREE(new_work_conn_request_message);
 	SAFE_FREE(work_c);
@@ -229,6 +230,15 @@ set_ticker_ping_timer(struct event *timeout)
 	event_add(timeout, &tv);
 }
 
+static void
+set_tcp_mux_ping_timer(struct event *timeout)
+{
+	struct timeval tv;
+	evutil_timerclear(&tv);
+	tv.tv_sec = 60;
+	event_add(timeout, &tv);
+}
+
 static void 
 hb_sender_cb(evutil_socket_t fd, short event, void *arg)
 {
@@ -241,6 +251,13 @@ hb_sender_cb(evutil_socket_t fd, short event, void *arg)
 	set_ticker_ping_timer(main_ctl->ticker_ping);	
 }
 
+static void
+tcp_mux_hb_sender_cb(evutil_socket_t fd, short event, void *arg)
+{
+	tcp_mux_send_ping(main_ctl->connect_bev, main_ctl->tcp_mux_ping_id++);
+	set_tcp_mux_ping_timer(main_ctl->tcp_mux_ping_event);
+}
+
 // return: 0: raw succeed 1: raw failed
 static int 
 proxy_service_resp_raw(struct new_proxy_response *npr)
@@ -286,29 +303,15 @@ proxy_service_resp_raw(struct new_proxy_response *npr)
 	return 0;
 }
 
-static void
-dump_all_unknown_encypt(uint8_t *enc_msg, int ilen)
-{
-	uint8_t *iv 	= get_main_decoder()->iv;
-	uint8_t *key 	= get_main_decoder()->key;
-	FILE *fout = fopen("unkown.encrypt", "w");
-	assert(fout);
-	fwrite(iv, 16, 1, fout);	
-	fwrite(key, 16, 1, fout);
-	fwrite((uint8_t *)&ilen, sizeof(int), 1, fout);
-	fwrite(enc_msg, ilen, 1, fout);
-	fclose(fout);
-}
-
 static int
-handle_enc_msg(uint8_t *enc_msg, int ilen, uint8_t **out)
+handle_enc_msg(const uint8_t *enc_msg, int ilen, uint8_t **out)
 {
 	if (ilen <= 0) {
 		debug(LOG_INFO, "enc_msg length should not be %d", ilen);
 		return -1;
 	}
 
-	uint8_t *buf = enc_msg;
+	const uint8_t *buf = enc_msg;
 	if ( !is_decoder_inited() && get_block_size() <= ilen) {
 		init_main_decoder(buf);
 		buf += get_block_size();
@@ -329,29 +332,20 @@ handle_enc_msg(uint8_t *enc_msg, int ilen, uint8_t **out)
 	return len;	
 }
 
-static void
-print_enc_msg(uint8_t *enc_msg, int len)
-{
-	printf("enc_msg is [%d]:\n", len);
-	for (int i = 0; i < len; i++) {
-		if (i%16 == 0)
-			printf("\n");
-		printf("%1x ", enc_msg[i]);
-	}
-	printf("\n");
-}
-
 static void
 handle_control_work(const uint8_t *buf, int len, void *ctx)
 {
 	uint8_t *frps_cmd = NULL;
 	uint8_t cmd_type;
-	uint8_t *enc_msg = buf;
+	const uint8_t *enc_msg = buf;
 
-	if (!ctx)	
+	if (!ctx) {	
+		debug(LOG_DEBUG, "main control message");
 		handle_enc_msg(enc_msg, len, &frps_cmd);
-	else
-		frps_cmd = buf;
+	} else {
+		debug(LOG_DEBUG, "worker message");
+		frps_cmd = (uint8_t *)buf;
+	}
 
 	if (!frps_cmd)	
 		return; // only recv iv
@@ -359,7 +353,7 @@ handle_control_work(const uint8_t *buf, int len, void *ctx)
 	struct msg_hdr *msg = (struct msg_hdr *)frps_cmd;
 
 	cmd_type = msg->type;
-	debug(LOG_DEBUG, "cmd_type is %d data is %s", cmd_type, msg->data);
+	debug(LOG_DEBUG, "cmd_type is %c data is %s", cmd_type, msg->data);
 	switch(cmd_type) {
 	case TypeReqWorkConn: 
 		debug(LOG_DEBUG, "TypeReqWorkConn cmd");
@@ -371,7 +365,7 @@ handle_control_work(const uint8_t *buf, int len, void *ctx)
 		break;
 	case TypeNewProxyResp:
 		debug(LOG_DEBUG, "TypeNewProxyResp cmd");
-		struct new_proxy_response *npr = new_proxy_resp_unmarshal(msg->data);
+		struct new_proxy_response *npr = new_proxy_resp_unmarshal((const char *)msg->data);
 		if (npr == NULL) {
 			debug(LOG_ERR, "new proxy response buffer unmarshal faild!");
 			return;
@@ -382,7 +376,7 @@ handle_control_work(const uint8_t *buf, int len, void *ctx)
 		break;
 	case TypeStartWorkConn:
 		debug(LOG_DEBUG, "TypeStartWorkConn cmd");
-		struct start_work_conn_resp *sr = start_work_conn_resp_unmarshal(msg->data); 
+		struct start_work_conn_resp *sr = start_work_conn_resp_unmarshal((const char *)msg->data); 
 		if (! sr) {
 			debug(LOG_ERR, 
 				"TypeStartWorkConn unmarshal failed, it should never be happend!");
@@ -420,8 +414,6 @@ handle_control_work(const uint8_t *buf, int len, void *ctx)
 		break;
 	default:
 		debug(LOG_INFO, "command type dont support: ctx is %d", ctx?1:0);
-		print_enc_msg(enc_msg, len);
-		dump_all_unknown_encypt(enc_msg, len);
 	}
 	
 	if (!ctx)
@@ -437,7 +429,7 @@ handle_login_response(const uint8_t *buf, int len)
 		return 0;
 	}	
 	
-	struct login_resp *lres = login_resp_unmarshal(mhdr->data); 
+	struct login_resp *lres = login_resp_unmarshal((const char *)mhdr->data); 
 	if (!lres) {
 		return 0;
 	}
@@ -452,14 +444,18 @@ handle_login_response(const uint8_t *buf, int len)
 	is_login = 1;
 
 	int login_len = msg_hton(mhdr->length);
-	debug(LOG_ERR, "login success! %d len %d", login_len, len);
-	if (len-login_len-sizeof(struct msg_hdr) == 0)
+	int ilen = len - login_len - sizeof(struct msg_hdr);
+	debug(LOG_ERR, "login success! login_len %d len %d ilen %d", login_len, len, ilen);
+	assert(ilen >= 0);
+	if (ilen <= 0)
 		return 1;
 	
-	// in case, system get 3 packet together 
-	uint8_t *enc_msg = mhdr->data+login_len;
+	// in case, libevent reveive continue packet together
+	struct common_conf 	*c_conf = get_common_config();
+	assert(c_conf->tcp_mux == 0);
+	uint8_t *enc_msg = mhdr->data + login_len; 
 	uint8_t *frps_cmd = NULL;
-	int nret = handle_enc_msg(enc_msg, len-login_len-sizeof(struct msg_hdr), &frps_cmd);
+	int nret = handle_enc_msg(enc_msg, ilen, &frps_cmd);
 	assert(nret > 0);
 	// start proxy services must first send
 	start_proxy_services();
@@ -472,18 +468,12 @@ handle_login_response(const uint8_t *buf, int len)
 }
 
 static void
-handle_frps_msg(unsigned char *buf, int len, void *ctx)
+handle_frps_msg(uint8_t *buf, int len, void *ctx)
 {
 	if (!is_login) {
 		// login response
 		handle_login_response(buf, len);
-	}else if (!ctx) {
-		// control msg
-		debug(LOG_DEBUG, "main control message");
-		handle_control_work(buf, len, NULL);
 	}else {
-		// client msg
-		debug(LOG_DEBUG, "client message");
 		handle_control_work(buf, len, ctx);
 	}	
 }
@@ -499,12 +489,17 @@ recv_cb(struct bufferevent *bev, void *ctx)
 		return;
 	}
 
-	unsigned char *buf = calloc(len+1, 1);
+	uint8_t *buf = calloc(len+1, 1);
 	assert(buf);
 	evbuffer_remove(input, buf, len);
-	debug(LOG_DEBUG, "recv msg from frps %d ", len);
+
+	struct common_conf 	*c_conf = get_common_config();
 	
-	handle_frps_msg(buf, len, ctx);
+	if (c_conf->tcp_mux) {
+		handle_tcp_mux_frps_msg(buf, len, handle_frps_msg);
+	} else {
+		handle_frps_msg(buf, len, ctx);
+	}
 
 	SAFE_FREE(buf);
 	return;
@@ -514,26 +509,27 @@ static void
 connect_event_cb (struct bufferevent *bev, short what, void *ctx)
 {
 	struct common_conf 	*c_conf = get_common_config();
-	static int retry_times = 0;
+	static int retry_times = 1;
 	if (what & (BEV_EVENT_EOF|BEV_EVENT_ERROR)) {
-		if (retry_times >= 10) { // only try 10 times consecutively
+		if (retry_times >= 100) {
 			debug(LOG_ERR, 
 				"have retry connect to xfrp server for %d times, exit!", 
 				retry_times);
 
 			exit(0);
 		}
-
+		sleep(retry_times);
 		retry_times++;
-		debug(LOG_ERR, "error: connect server [%s:%d] failed", 
+		debug(LOG_ERR, "error: connect server [%s:%d] failed %s", 
 				c_conf->server_addr, 
-				c_conf->server_port);
-		free_control();
-		init_main_control();
+				c_conf->server_port,
+				strerror(errno));
+		clear_main_control();
 		start_base_connect();
-		close_main_control();
 	} else if (what & BEV_EVENT_CONNECTED) {
 		retry_times = 0;
+
+		tcp_mux_send_win_update_syn(bev, main_ctl->stream_id);	
 		login();
 	}
 }
@@ -542,13 +538,24 @@ static void
 keep_control_alive() 
 {
 	main_ctl->ticker_ping = evtimer_new(main_ctl->connect_base, hb_sender_cb, NULL);
-	if ( ! main_ctl->ticker_ping) {
+	if ( !main_ctl->ticker_ping) {
 		debug(LOG_ERR, "Ping Ticker init failed!");
 		return;
 	}
 	set_ticker_ping_timer(main_ctl->ticker_ping);
 }
 
+static void
+keep_control_tcp_mux_alive()
+{
+	struct common_conf *c_conf = get_common_config();
+	if (!c_conf->tcp_mux) return;
+
+	main_ctl->tcp_mux_ping_event = evtimer_new(main_ctl->connect_base, tcp_mux_hb_sender_cb, NULL);
+	assert(main_ctl->tcp_mux_ping_event);
+	set_tcp_mux_ping_timer(main_ctl->tcp_mux_ping_event);
+}
+
 static void 
 server_dns_cb(int event_code, struct evutil_addrinfo *addr, void *ctx)
 {
@@ -579,11 +586,15 @@ void
 start_base_connect()
 {
 	struct common_conf *c_conf = get_common_config();
+	if (main_ctl->connect_bev)
+		bufferevent_free(main_ctl->connect_bev);
+
 	main_ctl->connect_bev = connect_server(main_ctl->connect_base, 
 						c_conf->server_addr, 
 						c_conf->server_port);
 	if ( ! main_ctl->connect_bev) {
-		debug(LOG_ERR, "error: connect server [%s:%d] failed", c_conf->server_addr, c_conf->server_port);
+		debug(LOG_ERR, "error: connect server [%s:%d] failed: [%d: %s]", 
+						c_conf->server_addr, c_conf->server_port, errno, strerror(errno));
 		exit(0);
 	}
 
@@ -603,7 +614,7 @@ login()
 		exit(0);
 	}
 	
-	send_msg_frp_server(NULL, TypeLogin, lg_msg, len, main_ctl->session_id);
+	send_msg_frp_server(NULL, TypeLogin, lg_msg, len, 1);
 	SAFE_FREE(lg_msg);
 }
 
@@ -622,15 +633,20 @@ send_msg_frp_server(struct bufferevent *bev,
 	}
 	assert(bout);
 
+	
+
 	debug(LOG_DEBUG, "send plain msg ----> [%c: %s]", type, msg);
 	
-	struct msg_hdr *req_msg = calloc(msg_len+sizeof(struct msg_hdr), 1);
+	size_t len = msg_len + sizeof(struct msg_hdr);
+	struct msg_hdr *req_msg = calloc(len, 1);
 	assert(req_msg);
 	req_msg->type = type;
 	req_msg->length = msg_hton((uint64_t)msg_len);
 	memcpy(req_msg->data, msg, msg_len);
 	
-	bufferevent_write(bout, (uint8_t *)req_msg, msg_len+sizeof(struct msg_hdr));
+	tcp_mux_send_data(bout, sid, len);
+
+	bufferevent_write(bout, (uint8_t *)req_msg, len);
 	
 	free(req_msg);
 }
@@ -660,6 +676,9 @@ send_enc_msg_frp_server(struct bufferevent *bev,
 
 	if (get_main_encoder() == NULL) {
 		debug(LOG_DEBUG, "init_main_encoder .......");
+		
+		tcp_mux_send_data(bout, sid, 16);
+
 		struct frp_coder *coder = init_main_encoder();
 		bufferevent_write(bout, coder->iv, 16);
 	}
@@ -669,6 +688,8 @@ send_enc_msg_frp_server(struct bufferevent *bev,
 	assert(olen > 0);
 	debug(LOG_DEBUG, "encrypt_data length %d", olen);
 
+	tcp_mux_send_data(bout, sid, olen);
+
 	bufferevent_write(bout, enc_msg, olen);
 
 	free(enc_msg);	
@@ -717,7 +738,7 @@ send_new_proxy(struct proxy_service *ps)
 
 	debug(LOG_DEBUG, "control proxy client: [Type %d : proxy_name %s : msg_len %d]", TypeNewProxy, ps->proxy_name, len);
 
-	send_enc_msg_frp_server(NULL, TypeNewProxy, new_proxy_msg, len, main_ctl->session_id);
+	send_enc_msg_frp_server(NULL, TypeNewProxy, new_proxy_msg, len, main_ctl->stream_id);
 	SAFE_FREE(new_proxy_msg);
 }
 
@@ -733,14 +754,6 @@ init_main_control()
 	assert(main_ctl);
 
 	struct common_conf *c_conf = get_common_config();
-	if (c_conf->tcp_mux) {
-		uint32_t *sid = init_sid_index();
-		assert(sid);
-		main_ctl->session_id = *sid;
-
-		debug(LOG_DEBUG, "Connect Frps with control session ID: %d", main_ctl->session_id);
-	}
-
 	struct event_base *base = NULL;
 	struct evdns_base *dnsbase = NULL; 
 	base = event_base_new();
@@ -749,6 +762,13 @@ init_main_control()
 		exit(0);
 	}
 	main_ctl->connect_base = base;
+	
+	if (c_conf->tcp_mux)
+		main_ctl->stream_id = get_next_session_id();
+	
+	// if server_addr is ip, done control init.
+	if (is_valid_ip_address((const char *)c_conf->server_addr))
+		return;
 
 	dnsbase = evdns_base_new(base, 1);
 	if (! dnsbase) {
@@ -767,10 +787,7 @@ init_main_control()
     evdns_base_nameserver_ip_add(dnsbase, "223.6.6.6");			//AliDNS
 	evdns_base_nameserver_ip_add(dnsbase, "114.114.114.114");	//114DNS
 
-	// if server_addr is ip, done control init.
-	if (is_valid_ip_address((const char *)c_conf->server_addr))
-		return;
-	
+		
 	// if server_addr is domain, analyze it to ip for server_ip
 	debug(LOG_DEBUG, "Get ip address of [%s] from DNServer", c_conf->server_addr);
 
@@ -788,19 +805,38 @@ init_main_control()
 							&hints, 
 							server_dns_cb, 
 							NULL);
-	if (! dns_req) {
+	if (!dns_req) {
 		debug(LOG_ERR, "error: can not analyse the dns of [%s]", c_conf->server_addr);
 		exit(0);
 	}
 }
 
+static void 
+free_main_control()
+{
+	SAFE_FREE(main_ctl);
+	main_ctl = NULL;
+}
+
+static void
+clear_main_control()
+{
+	assert(main_ctl);
+	if (main_ctl->ticker_ping) evtimer_del(main_ctl->ticker_ping);
+	if (main_ctl->tcp_mux_ping_event) evtimer_del(main_ctl->tcp_mux_ping_event);
+	clear_all_proxy_client();
+}
+
 void 
 close_main_control()
 {
-	assert(main_ctl);
+	clear_main_control();
+
 	event_base_dispatch(main_ctl->connect_base);
-	event_base_free(main_ctl->connect_base);
 	evdns_base_free(main_ctl->dnsbase, 0);
+	event_base_free(main_ctl->connect_base);
+
+	free_main_control();
 }
 
 void 
@@ -808,13 +844,7 @@ run_control()
 {
 	start_base_connect();
 	keep_control_alive();
+	keep_control_tcp_mux_alive();
 }
 
-void 
-free_control()
-{
-	if (!main_ctl)
-		return;
 
-	SAFE_FREE(main_ctl);
-}
