@@ -1,3 +1,30 @@
+/* vim: set et ts=4 sts=4 sw=4 : */
+/********************************************************************\
+ * This program is free software; you can redistribute it and/or    *
+ * modify it under the terms of the GNU General Public License as   *
+ * published by the Free Software Foundation; either version 2 of   *
+ * the License, or (at your option) any later version.              *
+ *                                                                  *
+ * This program is distributed in the hope that it will be useful,  *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
+ * GNU General Public License for more details.                     *
+ *                                                                  *
+ * You should have received a copy of the GNU General Public License*
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
+ * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ *                                                                  *
+\********************************************************************/
+
+/** @file crypto.c
+    @brief xfrpc crypto implement
+    @author Copyright (C) 2016 Dengfeng Liu <liu_df@qq.com>
+*/
+
+
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -67,7 +94,7 @@ init_main_encoder()
 }
 
 struct frp_coder *
-init_main_decoder(unsigned char *iv)
+init_main_decoder(const uint8_t *iv)
 {
 	struct common_conf *c_conf = get_common_config();
 	main_decoder = new_coder(c_conf->auth_token, default_salt);
@@ -132,19 +159,11 @@ encrypt_iv(unsigned char *iv_buf, size_t iv_len)
 	return iv_buf;
 }
 
-static void
-print_hex(uint8_t *val, size_t len)
-{
-	for(int i = 0; i < len; i++)
-		printf("%1x", val[i]);
-	printf("\n");
-}
-
 // using aes-128-cfb and nopadding
 size_t 
-encrypt_data(const unsigned char *src_data, size_t srclen, struct frp_coder *encoder, unsigned char **ret)
+encrypt_data(const uint8_t *src_data, size_t srclen, struct frp_coder *encoder, unsigned char **ret)
 {
-	uint8_t *intext = src_data;
+	uint8_t *intext = (uint8_t *)src_data;
 	assert(intext);
 	assert(encoder);
 	struct frp_coder *c = encoder;
@@ -180,7 +199,7 @@ E_END:
 size_t 
 decrypt_data(const uint8_t *enc_data, size_t enclen, struct frp_coder *decoder, uint8_t **ret)
 {
-	uint8_t *inbuf = enc_data;
+	uint8_t *inbuf = (uint8_t *)enc_data;
 	uint8_t *outbuf = calloc(enclen+1, 1);
 	struct frp_coder *c = decoder;
 	assert(inbuf);
