@@ -20,7 +20,7 @@
 \********************************************************************/
 
 /** @file msg.h
-    @brief xfrp msg struct
+    @brief xfrpc msg struct
     @author Copyright (C) 2016 Dengfeng Liu <liu_df@qq.com>
 */
 
@@ -114,4 +114,6 @@ int new_work_conn_marshal(const struct work_conn *work_c, char **msg);
 
 void control_response_free(struct control_response *res);
 
+char *get_msg_type(uint8_t type);
+
 #endif //_MSG_H_
